@@ -1,6 +1,6 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
  | Copyright (C) 2005-2012, The Roundcube Dev Team                       |
@@ -42,13 +42,13 @@ abstract class rcube_storage
      */
     public static $folder_types = array('drafts', 'sent', 'junk', 'trash');
 
-    protected $folder = 'INBOX';
+    protected $folder          = 'INBOX';
     protected $default_charset = 'ISO-8859-1';
+    protected $options         = array('auth_type' => 'check', 'language' => 'en_US');
+    protected $page_size       = 10;
+    protected $list_page       = 1;
+    protected $threading       = false;
     protected $search_set;
-    protected $options = array('auth_type' => 'check');
-    protected $page_size = 10;
-    protected $list_page = 1;
-    protected $threading = false;
 
     /**
      * All (additional) headers used (in any way) by Roundcube
@@ -94,13 +94,11 @@ abstract class rcube_storage
      */
     abstract function connect($host, $user, $pass, $port = 143, $use_ssl = null);
 
-
     /**
      * Close connection. Usually done on script shutdown
      */
     abstract function close();
 
-
     /**
      * Checks connection state.
      *
@@ -108,7 +106,6 @@ abstract class rcube_storage
      */
     abstract function is_connected();
 
-
     /**
      * Check connection state, connect if not connected.
      *
@@ -116,7 +113,6 @@ abstract class rcube_storage
      */
     abstract function check_connection();
 
-
     /**
      * Returns code of last error
      *
@@ -124,7 +120,6 @@ abstract class rcube_storage
      */
     abstract function get_error_code();
 
-
     /**
      * Returns message of last error
      *
@@ -132,7 +127,6 @@ abstract class rcube_storage
      */
     abstract function get_error_str();
 
-
     /**
      * Returns code of last command response
      *
@@ -140,7 +134,6 @@ abstract class rcube_storage
      */
     abstract function get_response_code();
 
-
     /**
      * Set connection and class options
      *
@@ -151,7 +144,6 @@ abstract class rcube_storage
         $this->options = array_merge($this->options, (array)$opt);
     }
 
-
     /**
      * Get connection/class option
      *
@@ -164,7 +156,6 @@ abstract class rcube_storage
         return $this->options[$name];
     }
 
-
     /**
      * Activate/deactivate debug mode.
      *
@@ -172,7 +163,6 @@ abstract class rcube_storage
      */
     abstract function set_debug($dbg = true);
 
-
     /**
      * Set default message charset.
      *
@@ -185,10 +175,9 @@ abstract class rcube_storage
         $this->default_charset = $cs;
     }
 
-
     /**
      * Set internal folder reference.
-     * All operations will be perfomed on this folder.
+     * All operations will be performed on this folder.
      *
      * @param  string $folder  Folder name
      */
@@ -201,7 +190,6 @@ abstract class rcube_storage
         $this->folder = $folder;
     }
 
-
     /**
      * Returns the currently used folder name
      *
@@ -212,7 +200,6 @@ abstract class rcube_storage
         return $this->folder;
     }
 
-
     /**
      * Set internal list page number.
      *
@@ -225,7 +212,6 @@ abstract class rcube_storage
         }
     }
 
-
     /**
      * Gets internal list page number.
      *
@@ -236,7 +222,6 @@ abstract class rcube_storage
         return $this->list_page;
     }
 
-
     /**
      * Set internal page size
      *
@@ -247,7 +232,6 @@ abstract class rcube_storage
         $this->page_size = (int) $size;
     }
 
-
     /**
      * Get internal page size
      *
@@ -258,7 +242,6 @@ abstract class rcube_storage
         return $this->page_size;
     }
 
-
     /**
      * Save a search result for future message listing methods.
      *
@@ -266,7 +249,6 @@ abstract class rcube_storage
      */
     abstract function set_search_set($set);
 
-
     /**
      * Return the saved search set.
      *
@@ -274,7 +256,6 @@ abstract class rcube_storage
      */
     abstract function get_search_set();
 
-
     /**
      * Returns the storage server's (IMAP) capability
      *
@@ -284,7 +265,6 @@ abstract class rcube_storage
      */
     abstract function get_capability($cap);
 
-
     /**
      * Sets threading flag to the best supported THREAD algorithm.
      * Enable/Disable threaded mode.
@@ -307,7 +287,6 @@ abstract class rcube_storage
         return $this->threading;
     }
 
-
     /**
      * Get current threading flag.
      *
@@ -318,7 +297,6 @@ abstract class rcube_storage
         return $this->threading;
     }
 
-
     /**
      * Checks the PERMANENTFLAGS capability of the current folder
      * and returns true if the given flag is supported by the server.
@@ -329,7 +307,6 @@ abstract class rcube_storage
      */
     abstract function check_permflag($flag);
 
-
     /**
      * Returns the delimiter that is used by the server
      * for folder hierarchy separation.
@@ -338,7 +315,6 @@ abstract class rcube_storage
      */
     abstract function get_hierarchy_delimiter();
 
-
     /**
      * Get namespace
      *
@@ -348,7 +324,6 @@ abstract class rcube_storage
      */
     abstract function get_namespace($name = null);
 
-
     /**
      * Get messages count for a specific folder.
      *
@@ -362,7 +337,6 @@ abstract class rcube_storage
      */
     abstract function count($folder = null, $mode = 'ALL', $force = false, $status = true);
 
-
     /**
      * Public method for listing message flags
      *
@@ -374,7 +348,6 @@ abstract class rcube_storage
      */
     abstract function list_flags($folder, $uids, $mod_seq = null);
 
-
     /**
      * Public method for listing headers.
      *
@@ -388,7 +361,6 @@ abstract class rcube_storage
      */
     abstract function list_messages($folder = null, $page = null, $sort_field = null, $sort_order = null, $slice = 0);
 
-
     /**
      * Return sorted list of message UIDs
      *
@@ -400,7 +372,6 @@ abstract class rcube_storage
      */
     abstract function index($folder = null, $sort_field = null, $sort_order = null);
 
-
     /**
      * Invoke search request to the server.
      *
@@ -413,7 +384,6 @@ abstract class rcube_storage
      */
     abstract function search($folder = null, $str = 'ALL', $charset = null, $sort_field = null);
 
-
     /**
      * Direct (real and simple) search request (without result sorting and caching).
      *
@@ -424,7 +394,6 @@ abstract class rcube_storage
      */
     abstract function search_once($folder = null, $str = 'ALL');
 
-
     /**
      * Refresh saved search set
      *
@@ -448,7 +417,6 @@ abstract class rcube_storage
      */
     abstract function get_message($uid, $folder = null);
 
-
     /**
      * Return message headers object of a specific message
      *
@@ -460,14 +428,13 @@ abstract class rcube_storage
      */
     abstract function get_message_headers($uid, $folder = null, $force = false);
 
-
     /**
      * Fetch message body of a specific message from the server
      *
      * @param  int                $uid    Message UID
      * @param  string             $part   Part number
      * @param  rcube_message_part $o_part Part object created by get_structure()
-     * @param  mixed              $print  True to print part, ressource to write part contents in
+     * @param  mixed              $print  True to print part, resource to write part contents in
      * @param  resource           $fp     File pointer to save the message part
      * @param  boolean            $skip_charset_conv Disables charset conversion
      *
@@ -475,7 +442,6 @@ abstract class rcube_storage
      */
     abstract function get_message_part($uid, $part = 1, $o_part = null, $print = null, $fp = null, $skip_charset_conv = false);
 
-
     /**
      * Fetch message body of a specific message from the server
      *
@@ -488,30 +454,29 @@ abstract class rcube_storage
     {
         $headers = $this->get_message_headers($uid);
         return rcube_charset::convert($this->get_message_part($uid, $part, null),
-            $headers->charset ? $headers->charset : $this->default_charset);
+            $headers->charset ?: $this->default_charset);
     }
 
-
     /**
      * Returns the whole message source as string (or saves to a file)
      *
-     * @param int      $uid Message UID
-     * @param resource $fp  File pointer to save the message
+     * @param int      $uid  Message UID
+     * @param resource $fp   File pointer to save the message
+     * @param string   $part Optional message part ID
      *
      * @return string Message source string
      */
-    abstract function get_raw_body($uid, $fp = null);
-
+    abstract function get_raw_body($uid, $fp = null, $part = null);
 
     /**
      * Returns the message headers as string
      *
-     * @param int $uid  Message UID
+     * @param int    $uid  Message UID
+     * @param string $part Optional message part ID
      *
      * @return string Message headers string
      */
-    abstract function get_raw_headers($uid);
-
+    abstract function get_raw_headers($uid, $part = null);
 
     /**
      * Sends the whole message source to stdout
@@ -521,7 +486,6 @@ abstract class rcube_storage
      */
     abstract function print_raw_body($uid, $formatted = true);
 
-
     /**
      * Set message flag to one or several messages
      *
@@ -534,7 +498,6 @@ abstract class rcube_storage
      */
     abstract function set_flag($uids, $flag, $folder = null, $skip_cache = false);
 
-
     /**
      * Remove message flag for one or several messages
      *
@@ -550,7 +513,6 @@ abstract class rcube_storage
         return $this->set_flag($uids, 'UN'.$flag, $folder);
     }
 
-
     /**
      * Append a mail message (source) to a specific folder.
      *
@@ -566,7 +528,6 @@ abstract class rcube_storage
      */
     abstract function save_message($folder, &$message, $headers = '', $is_file = false, $flags = array(), $date = null);
 
-
     /**
      * Move message(s) from one folder to another.
      *
@@ -578,7 +539,6 @@ abstract class rcube_storage
      */
     abstract function move_message($uids, $to, $from = null);
 
-
     /**
      * Copy message(s) from one mailbox to another.
      *
@@ -590,7 +550,6 @@ abstract class rcube_storage
      */
     abstract function copy_message($uids, $to, $from = null);
 
-
     /**
      * Mark message(s) as deleted and expunge.
      *
@@ -601,7 +560,6 @@ abstract class rcube_storage
      */
     abstract function delete_message($uids, $folder = null);
 
-
     /**
      * Expunge message(s) and clear the cache.
      *
@@ -613,7 +571,6 @@ abstract class rcube_storage
      */
     abstract function expunge_message($uids, $folder = null, $clear_cache = true);
 
-
     /**
      * Parse message UIDs input
      *
@@ -651,7 +608,7 @@ abstract class rcube_storage
 
 
     /* --------------------------------
-     *        folder managment
+     *        folder management
      * --------------------------------*/
 
     /**
@@ -667,7 +624,6 @@ abstract class rcube_storage
      */
     abstract function list_folders_subscribed($root = '', $name = '*', $filter = null, $rights = null, $skip_sort = false);
 
-
     /**
      * Get a list of all folders available on the server.
      *
@@ -681,7 +637,6 @@ abstract class rcube_storage
      */
     abstract function list_folders($root = '', $name = '*', $filter = null, $rights = null, $skip_sort = false);
 
-
     /**
      * Subscribe to a specific folder(s)
      *
@@ -691,7 +646,6 @@ abstract class rcube_storage
      */
     abstract function subscribe($folders);
 
-
     /**
      * Unsubscribe folder(s)
      *
@@ -701,7 +655,6 @@ abstract class rcube_storage
      */
     abstract function unsubscribe($folders);
 
-
     /**
      * Create a new folder on the server.
      *
@@ -712,7 +665,6 @@ abstract class rcube_storage
      */
     abstract function create_folder($folder, $subscribe = false);
 
-
     /**
      * Set a new name to an existing folder
      *
@@ -723,7 +675,6 @@ abstract class rcube_storage
      */
     abstract function rename_folder($folder, $new_name);
 
-
     /**
      * Remove a folder from the server.
      *
@@ -733,7 +684,6 @@ abstract class rcube_storage
      */
     abstract function delete_folder($folder);
 
-
     /**
      * Send expunge command and clear the cache.
      *
@@ -747,7 +697,6 @@ abstract class rcube_storage
         return $this->expunge_message('*', $folder, $clear_cache);
     }
 
-
     /**
      * Remove all messages in a folder..
      *
@@ -760,7 +709,6 @@ abstract class rcube_storage
         return $this->delete_message('*', $folder);
     }
 
-
     /**
      * Checks if folder exists and is subscribed
      *
@@ -771,7 +719,6 @@ abstract class rcube_storage
      */
     abstract function folder_exists($folder, $subscription = false);
 
-
     /**
      * Get folder size (size of all messages in a folder)
      *
@@ -781,7 +728,6 @@ abstract class rcube_storage
      */
     abstract function folder_size($folder);
 
-
     /**
      * Returns the namespace where the folder is in
      *
@@ -791,7 +737,6 @@ abstract class rcube_storage
      */
     abstract function folder_namespace($folder);
 
-
     /**
      * Gets folder attributes (from LIST response, e.g. \Noselect, \Noinferiors).
      *
@@ -802,7 +747,6 @@ abstract class rcube_storage
      */
     abstract function folder_attributes($folder, $force = false);
 
-
     /**
      * Gets connection (and current folder) data: UIDVALIDITY, EXISTS, RECENT,
      * PERMANENTFLAGS, UIDNEXT, UNSEEN
@@ -813,7 +757,6 @@ abstract class rcube_storage
      */
     abstract function folder_data($folder);
 
-
     /**
      * Returns extended information about the folder.
      *
@@ -823,7 +766,6 @@ abstract class rcube_storage
      */
     abstract function folder_info($folder);
 
-
     /**
      * Returns current status of a folder (compared to the last time use)
      *
@@ -834,7 +776,6 @@ abstract class rcube_storage
      */
     abstract function folder_status($folder = null, &$diff = array());
 
-
     /**
      * Synchronizes messages cache.
      *
@@ -842,7 +783,6 @@ abstract class rcube_storage
      */
     abstract function folder_sync($folder);
 
-
     /**
      * Modify folder name according to namespace.
      * For output it removes prefix of the personal namespace if it's possible.
@@ -856,7 +796,6 @@ abstract class rcube_storage
      */
     abstract function mod_folder($folder, $mode = 'out');
 
-
     /**
      * Create all folders specified as default
      */
@@ -877,7 +816,6 @@ abstract class rcube_storage
         }
     }
 
-
     /**
      * Check if specified folder is a special folder
      */
@@ -886,7 +824,6 @@ abstract class rcube_storage
         return $name == 'INBOX' || in_array($name, $this->get_special_folders());
     }
 
-
     /**
      * Return configured special folders
      */
@@ -907,17 +844,15 @@ abstract class rcube_storage
         return $this->icache['special-folders'];
     }
 
-
     /**
      * Set special folder associations stored in backend
      */
     public function set_special_folders($specials)
     {
-        // should be overriden by storage class if backend supports special folders (SPECIAL-USE)
+        // should be overridden by storage class if backend supports special folders (SPECIAL-USE)
         unset($this->icache['special-folders']);
     }
 
-
     /**
      * Get mailbox quota information.
      *
@@ -943,7 +878,6 @@ abstract class rcube_storage
      */
     abstract function set_acl($folder, $user, $acl);
 
-
     /**
      * Removes any <identifier,rights> pair for the
      * specified user from the ACL for the specified
@@ -956,7 +890,6 @@ abstract class rcube_storage
      */
     abstract function delete_acl($folder, $user);
 
-
     /**
      * Returns the access control list for a folder (GETACL).
      *
@@ -966,7 +899,6 @@ abstract class rcube_storage
      */
     abstract function get_acl($folder);
 
-
     /**
      * Returns information about what rights can be granted to the
      * user (identifier) in the ACL for the folder (LISTRIGHTS).
@@ -978,7 +910,6 @@ abstract class rcube_storage
      */
     abstract function list_rights($folder, $user);
 
-
     /**
      * Returns the set of rights that the current user has to a folder (MYRIGHTS).
      *
@@ -988,7 +919,6 @@ abstract class rcube_storage
      */
     abstract function my_rights($folder);
 
-
     /**
      * Sets metadata/annotations (SETMETADATA/SETANNOTATION)
      *
@@ -999,7 +929,6 @@ abstract class rcube_storage
      */
     abstract function set_metadata($folder, $entries);
 
-
     /**
      * Unsets metadata/annotations (SETMETADATA/SETANNOTATION)
      *
@@ -1010,18 +939,17 @@ abstract class rcube_storage
      */
     abstract function delete_metadata($folder, $entries);
 
-
     /**
      * Returns folder metadata/annotations (GETMETADATA/GETANNOTATION).
      *
      * @param string $folder   Folder name (empty for server metadata)
      * @param array  $entries  Entries
      * @param array  $options  Command options (with MAXSIZE and DEPTH keys)
+     * @param bool   $force    Disables cache use
      *
      * @return array Metadata entry-value hash array on success, NULL on error
      */
-    abstract function get_metadata($folder, $entries, $options = array());
-
+    abstract function get_metadata($folder, $entries, $options = array(), $force = false);
 
     /* -----------------------------------------
      *   Cache related functions
@@ -1036,7 +964,6 @@ abstract class rcube_storage
      */
     abstract function clear_cache($key = null, $prefix_mode = false);
 
-
     /**
      * Returns cached value
      *
@@ -1046,10 +973,8 @@ abstract class rcube_storage
      */
     abstract function get_cache($key);
 
-
     /**
      * Delete outdated cache entries
      */
     abstract function cache_gc();
-
-}  // end class rcube_storage
+}
